"use client";
import { useEffect, useState } from "react";
import { useStopwatch } from "react-timer-hook";
import Split from "react-split";

function ImageButton({ cards, i, handleClick }) {
  return (
    <button
      onClick={() => handleClick(i)}
      style={{ border: "none", background: "none", padding: 4 }}
    >
      <img
        src={
          cards[i] && (cards[i].isFlipped || cards[i].isMatched)
            ? `./images/Trump_${cards[i]?.number}.png`
            : "./images/Trump_Behind.png"
        }
        style={{ width: "100px", height: "100%" }}
        alt={`Card ${i}`}
      />
    </button>
  );
}

//ランダムにカードを配るための乱数作成
function Random() {
  //React Hydration エラー対策
  if (typeof window === "undefined") return []; // サーバーでは空の配列を返す
  // シャッフルする配列
  const arr = [1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10];
  let a = arr.length;

  // シャッフルアルゴリズム（Fisher-Yates）
  while (a) {
    const j = Math.floor(Math.random() * a);
    const t = arr[--a];
    arr[a] = arr[j];
    arr[j] = t;
  }
  return arr;
}

//カードの初期値を作成(ランダムな数を配る)
function Distribute() {
  const randomNumbers = Random(); // ランダムな数字の配列を取得

  // ランダムな数字を使用してinitialCardsを生成
  return randomNumbers.map((number, index) => ({
    id: index, //カードの連番
    number: number, //カードの数
    isFlipped: false, //裏か表
    isMatched: false, //数字が同じか
  }));
}

//ボードの表示
function Board({ cards, setCards, handleClick }) {
  return (
    <>
      <h1>{"一人神経衰弱"}</h1>
      <div className="board-row-images">
        <ImageButton
          handleClick={() => handleClick(0)}
          cards={cards}
          setCards={setCards}
          i={"0"}
        />
        <ImageButton
          handleClick={() => handleClick(1)}
          cards={cards}
          setCards={setCards}
          i={"1"}
        />
        <ImageButton
          handleClick={() => handleClick(2)}
          cards={cards}
          setCards={setCards}
          i={"2"}
        />
        <ImageButton
          handleClick={() => handleClick(3)}
          cards={cards}
          setCards={setCards}
          i={"3"}
        />
        <ImageButton
          handleClick={() => handleClick(4)}
          cards={cards}
          setCards={setCards}
          i={"4"}
        />
      </div>
      <div className="board-row-images">
        <ImageButton
          handleClick={() => handleClick(5)}
          cards={cards}
          setCards={setCards}
          i={"5"}
        />
        <ImageButton
          handleClick={() => handleClick(6)}
          cards={cards}
          setCards={setCards}
          i={"6"}
        />
        <ImageButton
          handleClick={() => handleClick(7)}
          cards={cards}
          setCards={setCards}
          i={"7"}
        />
        <ImageButton
          handleClick={() => handleClick(8)}
          cards={cards}
          setCards={setCards}
          i={"8"}
        />
        <ImageButton
          handleClick={() => handleClick(9)}
          cards={cards}
          setCards={setCards}
          i={"9"}
        />
      </div>
      <div className="board-row-images">
        <ImageButton
          handleClick={() => handleClick(10)}
          cards={cards}
          setCards={setCards}
          i={"10"}
        />
        <ImageButton
          handleClick={() => handleClick(11)}
          cards={cards}
          setCards={setCards}
          i={"11"}
        />
        <ImageButton
          handleClick={() => handleClick(12)}
          cards={cards}
          setCards={setCards}
          i={"12"}
        />
        <ImageButton
          handleClick={() => handleClick(13)}
          cards={cards}
          setCards={setCards}
          i={"13"}
        />
        <ImageButton
          handleClick={() => handleClick(14)}
          cards={cards}
          setCards={setCards}
          i={"14"}
        />
      </div>
      <div className="board-row-images">
        <ImageButton
          handleClick={() => handleClick(15)}
          cards={cards}
          setCards={setCards}
          i={"15"}
        />
        <ImageButton
          handleClick={() => handleClick(16)}
          cards={cards}
          setCards={setCards}
          i={"16"}
        />
        <ImageButton
          handleClick={() => handleClick(17)}
          cards={cards}
          setCards={setCards}
          i={"17"}
        />
        <ImageButton
          handleClick={() => handleClick(18)}
          cards={cards}
          setCards={setCards}
          i={"18"}
        />
        <ImageButton
          handleClick={() => handleClick(19)}
          cards={cards}
          setCards={setCards}
          i={"19"}
        />
      </div>
    </>
  );
}

function GameTimer({ isRunning, seconds, minutes, start, pause, reset }) {
  return (
    <div style={{ textAlign: "center" }}>
      <h1>Game Timer</h1>
      <div style={{ fontSize: "100px" }}>
        {minutes}:{seconds}
      </div>
      <p>{isRunning ? "Running" : "Paused"}</p>
      <button onClick={start}>Start</button>
      <button onClick={pause}>Stop</button>
      <button onClick={reset}>Retry</button>
    </div>
  );
}

export default function Game() {
  const [cards, setCards] = useState(
    Array(20).fill({
      id: 0,
      number: 0,
      isFlipped: false,
      isMatched: false,
    })
  );
  const [selectedCards, setSelectedCards] = useState([]); // 選択中のカード
  const [isProcessing, setIsProcessing] = useState(false); // 処理中かどうか
  const [isGameStarted, setIsGameStarted] = useState(false); //ゲームが始まっているか
  const { seconds, minutes, isRunning, start, pause, reset } = useStopwatch({
    autoStart: false,
  });
  const [gameState, setGameState] = useState("beforeStart"); // "beforeStart", "inProgress", "gameOver"

  useEffect(() => {
    // クライアントサイドでカードを生成
    setCards(Distribute());
  }, []);

  function handleClick(i) {
    if (!isGameStarted) {
      setIsGameStarted(true); // ゲームを開始状態に
      setGameState("inProgress"); // ゲーム中に設定
      start(); // タイマーを開始
    }
    // 処理中の場合はクリックを無視
    if (isProcessing) return;
    // すでにマッチ済みまたは裏返し中のカードは無視
    if (cards[i].isFlipped || cards[i].isMatched) {
      return;
    }

    const nextCards = [...cards]; // 配列をコピー
    nextCards[i].isFlipped = true; // カードを表にする
    setCards(nextCards);

    // 選択されたカードを更新
    const newSelectedCards = [...selectedCards, i];
    setSelectedCards(newSelectedCards);

    // 2枚目が選択された場合
    if (newSelectedCards.length === 2) {
      // 処理を開始
      setIsProcessing(true);

      const [first, second] = newSelectedCards;
      if (cards[first].number === cards[second].number) {
        // 数字が一致した場合
        setTimeout(() => {
          const matchedCards = [...nextCards];
          matchedCards[first].isMatched = true;
          matchedCards[second].isMatched = true;
          setCards(matchedCards);
          setSelectedCards([]); // 選択中カードをリセット
          setIsProcessing(false); // 処理完了
          // 全てのカードがマッチしたらタイマーを停止
          if (matchedCards.every((card) => card.isMatched)) {
            pause();
            setIsGameStarted(false);
            setGameState("gameOver"); // ゲーム終了に設定
            alert(`Game over! Time: ${minutes}:${seconds}`);
          }
        }, 500); // 少し間を置いて処理
      } else {
        // 数字が一致しない場合
        setTimeout(() => {
          const resetCards = [...nextCards];
          resetCards[first].isFlipped = false;
          resetCards[second].isFlipped = false;
          setCards(resetCards);
          setSelectedCards([]); // 選択中カードをリセット
          setIsProcessing(false); // 処理完了
        }, 1000); // 1秒間表示してから裏返す
      }
    }
  }

  function handleRetry() {
    setCards(Distribute());
    setSelectedCards([]);
    setIsProcessing(false);
    setIsGameStarted(false);
    reset(); // タイマーをリセット
    setGameState("beforeStart");
  }

  function handleStart() {
    setIsGameStarted(true);
    start();
  }
  /**/
  function Balloon({ gameState, minutes, seconds }) {
    const getComment = () => {
      if (gameState === "beforeStart") {
        return "startボタンかカードをクリックするとゲームを開始できるよ！";
      } else if (gameState === "inProgress") {
        return "カードをマッチさせよう！Stopボタンでゲームを中断できるよ！";
      } else if (gameState === "gameOver") {
        return (
          <span>
            お疲れ様です！クリアタイムは {minutes}:{seconds} でした！
            <br />
            Retryボタンでもう一度ゲームをはじめからできるよ！
          </span>
        );
      }
    };

    return (
      <Split
        sizes={[70, 30]} // 左: 吹き出し 70%, 右: 画像 30%
        gutterSize={0} // 分割線を非表示
        style={{
          display: "flex",
          flexDirection: "row",
          height: "150px", // 全体の高さを設定
        }}
      >
        {/** 左側のコンテンツ（吹き出し） */}
        <div
          style={{
            display: "flex",
            alignItems: "center",
            justifyContent: "flex-end",
            backgroundColor: "#fff",
            padding: "10px",
            borderRight: "1px solid #ccc",
          }}
        >
          <div
            style={{
              padding: "10px 20px",
              backgroundColor: "#f1f1f1",
              borderRadius: "15px",
              border: "1px solid #ccc",
              position: "relative",
            }}
          >
            <span style={{ fontSize: "18px", lineHeight: "1.2" }}>
              {getComment()}
            </span>
            <div
              style={{
                position: "absolute",
                top: "50%",
                right: "-10px", // 矢印を右端に
                marginTop: "-10px",
                width: "20px",
                height: "20px",
                backgroundColor: "#f1f1f1",
                borderBottom: "1px solid #ccc",
                borderRight: "1px solid #ccc",
                transform: "rotate(45deg)",
              }}
            ></div>
          </div>
        </div>

        {/** 右側のコンテンツ（画像） */}
        <div
          style={{
            display: "flex",
          }}
        >
          <img
            src="./images/Trump_man.png"
            alt="Trump Man"
            style={{
              width: "100px", // 画像の幅を固定
              height: "auto", // アスペクト比を保持
            }}
          />
        </div>
      </Split>
    );
  }

  return (
    <div className="game">
      <div className="game-board">
        <Board cards={cards} setCards={setCards} handleClick={handleClick} />
      </div>
      <div className="game-info">
        <Balloon gameState={gameState} minutes={minutes} seconds={seconds} />
        <GameTimer
          isRunning={isRunning}
          seconds={seconds}
          minutes={minutes}
          start={handleStart}
          pause={pause}
          reset={handleRetry}
        />
      </div>
    </div>
  );
}
